use crate::{
    bussy::PubSub,
    usernames::dto::UserBody,
    utils::{error::Error, structs::Subscription},
};

use nostr::{secp256k1::XOnlyPublicKey, Event, RelayMessage};
use nostr_database::Profile;
use std::{collections::BTreeSet, sync::Arc};

use super::user::Nip05Profile;

/// Handle core nostr events
pub trait Noose: Send + Sync {
    /// Start event listener
    async fn start(&mut self, pubsub: Arc<PubSub>) -> Result<(), Error>;

    /// Save event in the Database
    async fn write_event(&self, event: Box<Event>) -> Result<RelayMessage, Error>;

    /// Find events by subscription
    async fn find_event(&self, subscription: Subscription) -> Result<Vec<RelayMessage>, Error>;

    /// Get event counts by subscription
    async fn counts(&self, subscription: Subscription) -> Result<RelayMessage, Error>;

    /// Get NIP-05 of the registered User by 'username'
    async fn get_nip05(&self, username: String) -> Result<Nip05Profile, Error>;

    /// Create new NIP-05 for the User
    async fn create_nip05(&self, user: UserBody) -> Result<(), Error>;

    /// Get Profile by public key
    async fn profile(&self, public_key: XOnlyPublicKey) -> Result<Profile, Error>;

    /// Add new contact to Profile
    async fn add_contact(
        &self,
        profile_public_key: XOnlyPublicKey,
        contact_public_key: XOnlyPublicKey,
    ) -> Result<(), Error>;

    /// Remove contact from the Profile
    async fn remove_contact(
        &self,
        profile_public_key: XOnlyPublicKey,
        contact_public_key: XOnlyPublicKey,
    ) -> Result<(), Error>;

    /// Get Profile contats (pubkeys)
    async fn contacts_public_keys(
        &self,
        public_key: XOnlyPublicKey,
    ) -> Result<Vec<XOnlyPublicKey>, Error>;

    /// Get Profie contacts list
    async fn contacts(&self, public_key: XOnlyPublicKey) -> Result<BTreeSet<Profile>, Error>;
}
