use crate::utils::error::Error;
use validator::Validate;
use warp::{Filter, Rejection};

pub fn validate_body_filter<T: serde::de::DeserializeOwned + Send + Validate + 'static>(
) -> impl Filter<Extract = (T,), Error = Rejection> + Copy {
    warp::body::json::<T>().and_then(|query: T| async move {
        match query.validate() {
            Ok(_) => Ok(query),
            Err(e) => Err(warp::reject::custom(Error::validation_error(e))),
        }
    })
}

pub fn validate_query_filter<T: serde::de::DeserializeOwned + Send + Validate + 'static>(
) -> impl Filter<Extract = (T,), Error = Rejection> + Copy {
    warp::query::query::<T>().and_then(|query: T| async move {
        match query.validate() {
            Ok(_) => Ok(query),
            Err(e) => Err(warp::reject::custom(Error::validation_error(e))),
        }
    })
}
