use super::error::Error;

use anyhow::Result;
use serde::Serialize;
use warp::http::header::{HeaderValue, CONTENT_TYPE};

pub struct Response {
    pub body: Vec<u8>,
}

impl Response {
    pub fn new<T: Serialize>(body: &T) -> Result<Self, Error> {
        Ok(Self {
            body: serde_json::to_vec(body)?,
        })
    }
}

impl warp::Reply for Response {
    fn into_response(self) -> warp::reply::Response {
        let mut res = warp::reply::Response::new(self.body.into());
        let headers = res.headers_mut();

        headers.insert(CONTENT_TYPE, HeaderValue::from_static("application/json"));

        res
    }
}
